
import sys
import io
import os
from collections import OrderedDict
from attribute import *
import overlay 

class DtObject(object):
	"""
	This is the base object for most others.
	It shouldn't be created on its own.

	Args:
		parent: (:obj:`DtObject`)
			The Fragment or Device that will contain this Device
		name: (:obj:`str`)
			The name of the device.

	Returns:
		DtObject: A new DtObject object
	"""
	
	def __init__(self, parent, name):
		self.parent = parent
		self.name = name
		self.attributes = OrderedDict()
		self.objects = OrderedDict()
		if parent is not None: parent.addObject(self)

	def getParent(self, parent_class=None):
		"""
		Get the parent (containing) object of this object

		Args:
			parent_class: (:class:`DtObject`)
				The Fragment or Device that will contain this Device

		Returns:
			DtObject: The parent object
	"""
		
		if parent_class is None:
			return self.parent
			
		parent = self.parent
		while parent is not None:
			if type(parent) == parent_class:
				return parent
			parent = parent.parent
			
		return parent
		
	def getOverlay(self):
		return self.getParent(overlay.Overlay)

	def getDtsFile(self):
		return self.getParent(overlay.DtsFile)

	def addAttribute(self, attr, value=None):
		if type(attr) is list:
			for a in attr:
				self.attributes[a.name] = a
		elif issubclass(attr.__class__, Attribute):
			self.delAttribute(attr.name)
			self.attributes[attr.name] = attr
		else:
			self.delAttribute(attr)
			ao = Attribute(attr, value)
			self.attributes[ao.name] = ao

	def getAttribute(self, name):
		return self.attributes.get(name)
		
	def getAttributeValue(self, name):
		values = self.attributes.get(name).values
		if len(values) == 1:
			return values[0]
		return values
		
	def delAttribute(self, name):
		if name in self.attributes:
			del self.attributes[name]
		
	def addObject(self, obj):
		if type(obj) is list:
			for o in obj:
				self.objects[o.name] = o
		else:
			self.objects[obj.name] = obj

	def getObject(self, name):
		return self.objects.get(name)

	def delObject(self, name):
		if self.objects.get(name) is not None:
			del self.objects[name]

	def _print(self, indent, file_desc=sys.stdout):
		tabs = "".rjust(indent, '\t')
		file_desc.write("%s%s {\n" % (tabs, self.name))
		for aname, a in self.attributes.items():
			a._print(indent + 1, file_desc=file_desc)
		for oname, o in self.objects.items():
			o._print(indent + 1, file_desc=file_desc)
			
		tabs = "".rjust(indent, '\t')
		file_desc.write("%s};\n" % tabs)
		
		
		
		
