from device import *
from fragment import *

class PinConstants(object):
	TEGRA_PIN_ENABLE = "TEGRA_PIN_ENABLE"
	TEGRA_PIN_DISABLE = "TEGRA_PIN_DISABLE"
	TEGRA_PIN_PULL_UP = "TEGRA_PIN_PULL_UP"
	TEGRA_PIN_PULL_DOWN = "TEGRA_PIN_PULL_DOWN"
	TEGRA_PIN_PULL_NONE = "TEGRA_PIN_PULL_NONE"


class Pin(Device):
	"""
	Create a new Pin object.

	Args:
		parent: (:obj:`PinMux`)
			The PinMux object that will contain this pin
		name: (str)
			The name of the pin on the 40pin header.  E.G. "pin19".
		function: (str)
			The devicetree "function" that will claim the pin. E.G. "spi1".
		pins: (str)
			The internal pin name to assign to the physical pin. E.G. "spi1_mosi_pc0".
		pull: (:obj:`PinConstants`)
			PinConstants.TEGRA_PIN_PULL_UP, PinConstants.TEGRA_PIN_PULL_DOWN, PinConstants.TEGRA_PIN_PULL_NONE
		tristate: (bool)
			Should this pin be tristated?
		input: (bool)
			Should this pin be configured for input?

	Returns:
		Pin: A new pin object

	"""
	def __init__(self, parent, name, function, pins, pull, tristate, input):
		Device.__init__(self, parent, name)
		self.addAttribute(StringAttribute("nvidia,function", function))
		self.addAttribute(StringAttribute("nvidia,pins", pins))
		self.addAttribute(SymbolAttribute("nvidia,pull", pull))
		self.addAttribute(SymbolAttribute("nvidia,tristate", "TEGRA_PIN_ENABLE" if tristate else "TEGRA_PIN_DISABLE"))
		self.addAttribute(SymbolAttribute("nvidia,enable-input", "TEGRA_PIN_ENABLE" if input else "TEGRA_PIN_DISABLE"))

class PinMux(Fragment):
	"""
	Creates a new PinMux object for the 40 pin Jetson Nano header

	Args:
		parent : (:obj:`Overlay`)
			The Overlay object that will contain this PinMux object
		name : (str)
			A name for this PinMux object  
		target : (str)
			The target MUST be "pinmux" unless you are doing something special. 

	Returns:
		PinMux: A new PinMux object

	"""
	
	def __init__(self, parent, name, target="pinmux"):
		Fragment.__init__(self, parent, name, target)
		self.addAttribute(StringAttribute("pinctrl-names", "default"))
		self.addAttribute(AddrAttribute("pinctrl-0", "hdr40_pinmux"))
		self.addObject(Device(self, "hdr40_pinmux: header-40pin-pinmux"))
		
	def addPin(self, *args):
		"""
		Add a Pin to this PinMux object
		
		Args:
			pin: (:obj:`Pin`) A Pin object
			
		OR
		
		Args:
			parent : (:obj:`PinMux`)
				The PinMux object that will contain this pin
			name : (str)
				The name of the pin on the 40pin header.  E.G. "pin19".
			function : (str)
				The devicetree "function" that will claim the pin. E.G. "spi1".
			pins : (str)
				The internal pin name to assign to the physical pin. E.G. "spi1_mosi_pc0".
			pull : (:obj:`PinConstants`)
				PinConstants.TEGRA_PIN_PULL_UP, PinConstants.TEGRA_PIN_PULL_DOWN, PinConstants.TEGRA_PIN_PULL_NONE
			tristate: (bool)
				Should this pin be tristated?
			input: (bool)
				Should this pin be configured for input?
		
			
		"""
		pmdev = self.getObject("hdr40_pinmux: header-40pin-pinmux")
		if type(args[0]) == Pin:
			pmdev.addObject(args[0])
		else:
			pmdev.addObject(Pin(pmdev, *args))
